@@ -50,8 +50,19 @@
 #include "mkdtemp.h"
 #endif
 
+GQuark
+nautilus_burn_iso_error_quark (void)
+{
+	static GQuark quark = 0;
+	if (!quark)
+		quark = g_quark_from_static_string ("nautilus_burn_iso_error");
+
+	return quark;
+}
+
 static gboolean
-make_iso_get_free_space (const char *filename, GnomeVFSFileSize *size)
+make_iso_get_free_space (const char       *filename,
+			 GnomeVFSFileSize *size)
 {
 	GnomeVFSURI *uri;
 
@@ -75,6 +86,7 @@ struct mkisofs_output {
 	int         result;
 	int         pid;
 	const char *filename;
+	GError     *error;
 	gboolean    debug;
 };
 
@@ -89,13 +101,14 @@ static struct mkisofs_output *mkisofs_output_ptr;
 void
 nautilus_burn_make_iso_cancel (void)
 {
-	if (mkisofs_output_ptr)
-		{
-			kill (mkisofs_output_ptr->pid, SIGINT);
-			unlink (mkisofs_output_ptr->filename);
-
-			g_main_loop_quit (mkisofs_output_ptr->loop);
-		}
+	if (mkisofs_output_ptr) {
+		kill (mkisofs_output_ptr->pid, SIGINT);
+		unlink (mkisofs_output_ptr->filename);
+		mkisofs_output_ptr->error = g_error_new_literal (NAUTILUS_BURN_ISO_ERROR, 
+								 NAUTILUS_BURN_ISO_ERROR_GENERAL,
+								 _("The operation was cancelled by the user."));
+		g_main_loop_quit (mkisofs_output_ptr->loop);
+	}
 }
 
 static void
@@ -477,17 +490,28 @@ mkisofs_stdout_read (GIOChannel   *source,
 	struct mkisofs_output *mkisofs_output = data;
 	char *line;
 	GIOStatus status;
-	
-	status = g_io_channel_read_line (source,
-					 &line, NULL, NULL, NULL);
 
-	if (line && mkisofs_output->debug) {
-		g_print ("make_iso stdout: %s", line);
+	if (condition & G_IO_IN) {
+		status = g_io_channel_read_line (source,
+						 &line, NULL, NULL, NULL);
+
+		if (line && mkisofs_output->debug) {
+			g_print ("make_iso stdout: %s", line);
+		}
+
+		if (status == G_IO_STATUS_NORMAL) {
+			g_free (line);
+		} else if (status == G_IO_STATUS_EOF) {
+			if (mkisofs_output->debug)
+				g_print ("make_iso stdout: EOF\n");
+			return FALSE;
+		}
+	} else if (condition & G_IO_HUP) {
+		if (mkisofs_output->debug)
+			g_print ("make_iso stdout: HUP\n");
+		return FALSE;
 	}
 
-	if (status == G_IO_STATUS_NORMAL) {
-		g_free (line);
-	}
 	return TRUE;
 }
 
@@ -502,38 +526,105 @@ mkisofs_stderr_read (GIOChannel   *source,
 	double                 fraction;
 	GIOStatus              status;
 
-	status = g_io_channel_read_line (source, &line, NULL, NULL, NULL);
+	if (condition & G_IO_IN) {
+		status = g_io_channel_read_line (source, &line, NULL, NULL, NULL);
 
-	if (line && mkisofs_output->debug) {
-		g_print ("make_iso stderr: %s", line);
-	}
-
-	if (status == G_IO_STATUS_NORMAL) {
-		if (strncmp (line, "Total translation table size", 28) == 0) {
-			nautilus_burn_progress_set_fraction (1.0);
-			g_main_loop_quit (mkisofs_output->loop);
-			mkisofs_output->result = NAUTILUS_BURN_RECORDER_RESULT_FINISHED;
+		if (line && mkisofs_output->debug) {
+			g_print ("make_iso stderr: %s", line);
 		}
 
-		if (strstr (line, "estimate finish")) {
-			if (sscanf (line, "%6c%% done, estimate finish",
-				    fraction_str) == 1) {
-				fraction_str [6] = 0;
-				fraction = g_strtod (fraction_str, NULL);
-				nautilus_burn_progress_set_fraction (fraction/100.0);
+		if (status == G_IO_STATUS_NORMAL) {
+			if (strncmp (line, "Total translation table size", 28) == 0) {
+				nautilus_burn_progress_set_fraction (1.0);
+				mkisofs_output->result = NAUTILUS_BURN_RECORDER_RESULT_FINISHED;
 			}
-		}
-		if (strstr (line, "Incorrectly encoded string")) {
+
+			if (strstr (line, "estimate finish")) {
+				if (sscanf (line, "%6c%% done, estimate finish",
+					    fraction_str) == 1) {
+					fraction_str [6] = 0;
+					fraction = g_strtod (fraction_str, NULL);
+					nautilus_burn_progress_set_fraction (fraction/100.0);
+				}
+			}
+
+			if (strstr (line, "Incorrectly encoded string")) {
+				nautilus_burn_progress_set_fraction (1.0);
+				if (!mkisofs_output->error)
+					mkisofs_output->error = g_error_new_literal (NAUTILUS_BURN_ISO_ERROR,
+										     NAUTILUS_BURN_ISO_ERROR_GENERAL,
+										     _("Incorrectly encoded string."));
+				mkisofs_output->result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
+			}
+
+			if (strstr (line, "Unknown charset")) {
+				nautilus_burn_progress_set_fraction (1.0);
+				/* FIXME: use better error message after string freeze */
+				if (!mkisofs_output->error)
+					mkisofs_output->error = g_error_new_literal (NAUTILUS_BURN_ISO_ERROR,
+										     NAUTILUS_BURN_ISO_ERROR_GENERAL,
+										     _("Incorrectly encoded string."));
+				mkisofs_output->result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
+			}
+
+			if (strstr (line, "No space left on device")) {
+				nautilus_burn_progress_set_fraction (1.0);
+				if (!mkisofs_output->error)
+					mkisofs_output->error = g_error_new_literal (NAUTILUS_BURN_ISO_ERROR,
+										     NAUTILUS_BURN_ISO_ERROR_GENERAL,
+										     _("There is no space left on the device."));
+				mkisofs_output->result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
+			}
+
+			if (strstr (line, "Value too large for defined data type")) {
+				/* TODO: get filename from error message */
+				nautilus_burn_progress_set_fraction (1.0);
+				if (!mkisofs_output->error)
+					mkisofs_output->error = g_error_new_literal (NAUTILUS_BURN_ISO_ERROR,
+										     NAUTILUS_BURN_ISO_ERROR_GENERAL,
+										     _("File too large for filesystem."));
+				mkisofs_output->result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
+			}
+
+			g_free (line);
+		} else if (status == G_IO_STATUS_EOF) {
+			if (mkisofs_output->debug)
+				g_print ("make_iso stderr: EOF\n");
 			nautilus_burn_progress_set_fraction (1.0);
 			g_main_loop_quit (mkisofs_output->loop);
-			mkisofs_output->result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
+			return FALSE;
 		}
-
-		g_free (line);
+	} else if (condition & G_IO_HUP) {
+		/* only handle the HUP when we have read all available lines of output */
+		if (mkisofs_output->debug)
+			g_print ("make_iso stderr: HUP\n");
+		nautilus_burn_progress_set_fraction (1.0);
+		g_main_loop_quit (mkisofs_output->loop);
+		return FALSE;
 	}
+
 	return TRUE;
 }
 
+/* this is an ugly hack until utf8/iconv support is added into upstream mkisofs */
+static gboolean
+ncb_mkisofs_supports_utf8 (void)
+{
+	char    *standard_error;
+	gboolean supported;
+	gboolean res;
+
+	res = g_spawn_command_line_sync ("mkisofs -input-charset utf8", NULL, &standard_error, NULL, NULL);
+	if (res && !g_strrstr (standard_error, "Unknown charset"))
+		supported = TRUE;
+	else
+		supported = FALSE;
+
+	g_free (standard_error);
+
+	return supported;
+}
+
 /**
  * nautilus_burn_make_iso:
  * @filename:
@@ -551,7 +642,8 @@ nautilus_burn_make_iso (const char *filename,
 			const char *label,
 			gboolean    warn_low_space,
 			gboolean    use_joliet,
-			gboolean    debug)
+			gboolean    debug,
+			GError    **error)
 {
 	GnomeVFSURI          *uri;
 	char                 *filelist = NULL;
@@ -560,7 +652,7 @@ nautilus_burn_make_iso (const char *filename,
 	GList                *l;
 	int                   stdout_pipe, stderr_pipe;
 	int                   i;
-	GError               *error;
+	GError               *sub_error;
 	const char           *argv [20]; /* Shouldn't need more than 20 arguments */
 	struct mkisofs_output mkisofs_output;
 	GIOChannel           *channel;
@@ -568,14 +660,19 @@ nautilus_burn_make_iso (const char *filename,
 	char                 *stdout_data, *stderr_data;
 	char                 *dirname;
 	int                   exit_status, res;
-	unsigned long         iso_size;
-	GtkWidget            *dialog;
+	guint64               iso_size;
 	GnomeVFSFileSize      size;
+	gboolean              has_utf8_support;
 
-	if (label) {
-		g_return_val_if_fail (strlen (label) < 32, NAUTILUS_BURN_RECORDER_RESULT_ERROR);
+	if (label && (strlen (label) > 32)) {
+		g_set_error (error,
+			     G_FILE_ERROR,
+			     0,
+			     _("The label for the image is too long."));
+		return NAUTILUS_BURN_RECORDER_RESULT_ERROR;
 	}
 
+	mkisofs_output.error = NULL;
 	mkisofs_output.debug = debug;
 
 	dirname = g_strdup_printf ("iso-%s.XXXXXX", g_get_user_name ());
@@ -586,7 +683,10 @@ nautilus_burn_make_iso (const char *filename,
 	mkisofs_output.result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
 
 	if (state.tmpdir == 0) {
-		g_warning ("Unable to create temp dir\n");
+		mkisofs_output.error = g_error_new (NAUTILUS_BURN_ISO_ERROR,
+						    NAUTILUS_BURN_ISO_ERROR_GENERAL,
+						    _("Unable to create temporary directory: %s."),
+						    g_strerror (errno));
 		goto cleanup;
 	}
 	
@@ -598,6 +698,10 @@ nautilus_burn_make_iso (const char *filename,
 
 	state.graft_file = fopen (filelist, "w");
 	if (state.graft_file == NULL) {
+		mkisofs_output.error = g_error_new (NAUTILUS_BURN_ISO_ERROR, 
+						    NAUTILUS_BURN_ISO_ERROR_GENERAL,
+						    _("Unable to create temporary file: %s."),
+						    g_strerror (errno));
 		goto cleanup;
 	}
 	
@@ -610,21 +714,15 @@ nautilus_burn_make_iso (const char *filename,
 	state.remove_files = g_list_prepend (state.remove_files, g_strdup (filelist));
 
 	if (!state.found_file) {
-		dialog = gtk_message_dialog_new (nautilus_burn_progress_get_window (),
-						 GTK_DIALOG_DESTROY_WITH_PARENT,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
-						 _("No files selected"));
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-							  _("There are no files to write to disc."));
-		gtk_window_set_title (GTK_WINDOW (dialog),
-				      _("No files selected"));
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
+		mkisofs_output.error = g_error_new_literal (NAUTILUS_BURN_ISO_ERROR, 
+							    NAUTILUS_BURN_ISO_ERROR_GENERAL,
+							    _("There are no files to write to disc."));
 		mkisofs_output.result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
 		goto cleanup;
 	}
 
+	has_utf8_support = ncb_mkisofs_supports_utf8 ();
+
  retry:
 	i = 0;
 	argv [i++] = "mkisofs";
@@ -632,9 +730,10 @@ nautilus_burn_make_iso (const char *filename,
 	if (use_joliet) {
 		argv [i++] = "-J";
 	}
-	/* Undocumented -input-charset option */
-	argv [i++] = "-input-charset";
-	argv [i++] = "utf8";
+	if (has_utf8_support) {
+		argv [i++] = "-input-charset";
+		argv [i++] = "utf8";
+	}
 	argv [i++] = "-q";
 	argv [i++] = "-graft-points";
 	argv [i++] = "-path-list";
@@ -642,7 +741,7 @@ nautilus_burn_make_iso (const char *filename,
 	argv [i++] = "-print-size";
 	argv [i++] = NULL;
 
-	error = NULL;
+	sub_error = NULL;
 
 	if (debug) {
 		g_print ("launching command: ");
@@ -660,10 +759,12 @@ nautilus_burn_make_iso (const char *filename,
 			   &stdout_data,
 			   &stderr_data,
 			   &exit_status,
-			   &error)) {
-		g_warning ("mkisofs command failed: %s\n", error->message);
-		g_error_free (error);
-		/* TODO: Better error handling */
+			   &sub_error)) {
+		mkisofs_output.error = g_error_new (NAUTILUS_BURN_ISO_ERROR, 
+						    NAUTILUS_BURN_ISO_ERROR_GENERAL,
+						    _("Could not run sub process: %s."),
+						    sub_error->message);
+		g_error_free (sub_error);
 		mkisofs_output.result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
 		goto cleanup;
 	}
@@ -677,13 +778,16 @@ nautilus_burn_make_iso (const char *filename,
 				goto retry;
 			} else {
 				mkisofs_output.result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
+				mkisofs_output.error = g_error_new_literal (NAUTILUS_BURN_ISO_ERROR, 
+									    NAUTILUS_BURN_ISO_ERROR_GENERAL,
+									    _("The operation was cancelled by the user."));
 				goto cleanup;
 			}
 		}
 	}
 
 	g_free (stderr_data);
-	iso_size = atol (stdout_data); /* blocks of 2048 bytes */
+	iso_size = (guint64)atol (stdout_data) * 2048 ; /* mkisofs reports blocks of 2048 bytes */
 	g_free (stdout_data);
 
 	dirname = g_path_get_dirname (filename);
@@ -695,18 +799,10 @@ nautilus_burn_make_iso (const char *filename,
 	} else if (iso_size > size) {
 		g_free (dirname);
 		if (warn_low_space) {
-			dialog = gtk_message_dialog_new (nautilus_burn_progress_get_window (),
-							 GTK_DIALOG_DESTROY_WITH_PARENT,
-							 GTK_MESSAGE_ERROR,
-							 GTK_BUTTONS_CLOSE,
-							 _("Not enough space"));
-			gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-								  "Not enough space to store disc image (%ld MiB needed)",
-								  iso_size / 512);
-			gtk_window_set_title (GTK_WINDOW (dialog),
-					      _("Not enough space"));
-			gtk_dialog_run (GTK_DIALOG (dialog));
-			gtk_widget_destroy (dialog);
+			mkisofs_output.error = g_error_new (NAUTILUS_BURN_ISO_ERROR, 
+							    NAUTILUS_BURN_ISO_ERROR_GENERAL,
+							    _("The selected location does not have enough space to store the disc image (%ld MiB needed)."),
+							    (unsigned long)iso_size / 1048576);
 			mkisofs_output.result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
 		} else {
 			mkisofs_output.result = NAUTILUS_BURN_RECORDER_RESULT_RETRY;
@@ -721,8 +817,10 @@ nautilus_burn_make_iso (const char *filename,
 	if (use_joliet) {
 		argv [i++] = "-J";
 	}
-	argv [i++] = "-input-charset";
-	argv [i++] = "utf8";
+	if (has_utf8_support) {
+		argv [i++] = "-input-charset";
+		argv [i++] = "utf8";
+	}
 	argv [i++] = "-graft-points";
 	argv [i++] = "-path-list";
 	argv [i++] = filelist;
@@ -745,7 +843,7 @@ nautilus_burn_make_iso (const char *filename,
 	nautilus_burn_progress_set_text (_("Creating CD image"));
 	nautilus_burn_progress_set_fraction (0.0);
 	nautilus_burn_progress_set_image_spinning (TRUE);
-	error = NULL;
+	sub_error = NULL;
 	if (!g_spawn_async_with_pipes  (NULL,
 					(char **)argv,
 					NULL,
@@ -755,9 +853,12 @@ nautilus_burn_make_iso (const char *filename,
 					/*stdin*/NULL,
 					&stdout_pipe,
 					&stderr_pipe,
-					&error)) {
-		g_warning ("mkisofs command failed: %s\n", error->message);
-		g_error_free (error);
+					&sub_error)) {
+		mkisofs_output.error = g_error_new (NAUTILUS_BURN_ISO_ERROR, 
+						    NAUTILUS_BURN_ISO_ERROR_GENERAL,
+						    _("Command failed: %s"),
+						    sub_error->message);
+		g_error_free (sub_error);
 		mkisofs_output.result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
 		goto cleanup;
 	} else {
@@ -802,6 +903,15 @@ nautilus_burn_make_iso (const char *filename,
 	g_free (tempdir);
 	nautilus_burn_progress_set_image_spinning (FALSE);
 
+	if (mkisofs_output.result == NAUTILUS_BURN_RECORDER_RESULT_ERROR) {
+		if (!mkisofs_output.error)
+			mkisofs_output.error = g_error_new_literal (NAUTILUS_BURN_ISO_ERROR,
+								    NAUTILUS_BURN_ISO_ERROR_GENERAL,
+								    _("Unknown error"));
+
+		g_propagate_error (error, mkisofs_output.error);
+	}
+
 	return mkisofs_output.result;
 }
 
@@ -924,7 +1034,9 @@ nautilus_burn_verify_iso (const char *filename,
 	return TRUE;
 
  bail:
-	*error = g_error_new_literal (0, 0, _("Not a valid CD image."));
+	*error = g_error_new_literal (NAUTILUS_BURN_ISO_ERROR,
+				      NAUTILUS_BURN_ISO_ERROR_GENERAL,
+				      _("Not a valid CD image."));
 
 	return FALSE;
 }
