@@ -193,10 +193,11 @@ virtual_dir_lookup (VirtualNode *dir, const char *filename)
 static VirtualNode *
 virtual_node_lookup (VirtualRoot *root, const char *path, VirtualNode **parent)
 {
-	gchar *copy, *next;
+	gchar *copy, *next, *copy_orig;
 	VirtualNode *node;
 
-	copy = g_strdup (path);
+	copy_orig = g_strdup (path);
+	copy = copy_orig;
 
 	if (parent != NULL) {
 		*parent = NULL;
@@ -234,7 +235,8 @@ virtual_node_lookup (VirtualRoot *root, const char *path, VirtualNode **parent)
 
 		copy = next;
 	}
-	
+	g_free (copy_orig);
+
 	return node;
 }
 
